import { isArray, isInvalid, isNullOrUndef, isStatefulComponent, isUndefined, assign } from 'inferno-shared';
import { normalize } from './normalization';
import options from './options';
import { EMPTY_OBJ } from '../DOM/utils';
export function createVNode(flags, type, props, children, events, key, ref, noNormalise) {
    if (flags & 16 /* ComponentUnknown */) {
        flags = isStatefulComponent(type) ? 4 /* ComponentClass */ : 8 /* ComponentFunction */;
    }
    const vNode = {
        children: isUndefined(children) ? null : children,
        dom: null,
        events: events || null,
        flags,
        key: isUndefined(key) ? null : key,
        props: props || null,
        ref: ref || null,
        type
    };
    if (!noNormalise) {
        normalize(vNode);
    }
    if (options.createVNode) {
        options.createVNode(vNode);
    }
    return vNode;
}
export function cloneVNode(vNodeToClone, props) {
    let restParamLength = arguments.length - 2; // children
    let children;
    // Manually handle restParam for children, because babel always creates array
    // Not creating array allows us to fastPath out of recursion
    if (restParamLength > 0) {
        if (!props) {
            props = {};
        }
        if (restParamLength === 1) {
            children = arguments[2];
        }
        else {
            children = [];
            while (restParamLength-- > 0) {
                children[restParamLength] = arguments[restParamLength + 2];
            }
        }
        if (isUndefined(props.children)) {
            props.children = children;
        }
        else {
            if (isArray(children)) {
                if (isArray(props.children)) {
                    props.children = props.children.concat(children);
                }
                else {
                    props.children = [props.children].concat(children);
                }
            }
            else {
                if (isArray(props.children)) {
                    props.children.push(children);
                }
                else {
                    props.children = [props.children];
                    props.children.push(children);
                }
            }
        }
    }
    let newVNode;
    if (isArray(vNodeToClone)) {
        const tmpArray = [];
        for (let i = 0, len = vNodeToClone.length; i < len; i++) {
            tmpArray.push(cloneVNode(vNodeToClone[i]));
        }
        newVNode = tmpArray;
    }
    else {
        const flags = vNodeToClone.flags;
        const events = vNodeToClone.events || (props && props.events) || null;
        const key = !isNullOrUndef(vNodeToClone.key) ? vNodeToClone.key : (props ? props.key : null);
        const ref = vNodeToClone.ref || (props ? props.ref : null);
        if (flags & 28 /* Component */) {
            newVNode = createVNode(flags, vNodeToClone.type, (!vNodeToClone.props && !props) ? EMPTY_OBJ : assign({}, vNodeToClone.props, props), null, events, key, ref, true);
            const newProps = newVNode.props;
            if (newProps) {
                const newChildren = newProps.children;
                // we need to also clone component children that are in props
                // as the children may also have been hoisted
                if (newChildren) {
                    if (isArray(newChildren)) {
                        const len = newChildren.length;
                        if (len > 0) {
                            const tmpArray = [];
                            for (let i = 0; i < len; i++) {
                                const child = newChildren[i];
                                if (!isInvalid(child) && isVNode(child)) {
                                    tmpArray.push(cloneVNode(child));
                                }
                            }
                            newProps.children = tmpArray;
                        }
                    }
                    else if (isVNode(newChildren)) {
                        newProps.children = cloneVNode(newChildren);
                    }
                }
            }
            newVNode.children = null;
        }
        else if (flags & 3970 /* Element */) {
            children = (props && props.children) || vNodeToClone.children;
            newVNode = createVNode(flags, vNodeToClone.type, (!vNodeToClone.props && !props) ? EMPTY_OBJ : assign({}, vNodeToClone.props, props), children, events, key, ref, !children);
        }
        else if (flags & 1 /* Text */) {
            newVNode = createTextVNode(vNodeToClone.children, key);
        }
    }
    return newVNode;
}
export function createVoidVNode() {
    return createVNode(4096 /* Void */);
}
export function createTextVNode(text, key) {
    return createVNode(1 /* Text */, null, null, text, null, key);
}
export function isVNode(o) {
    return !!o.flags;
}
